package com.vendor.loader.util;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.loader.Log.Log;
import com.vendor.loader.Messages;
import com.vendor.loader.sparse.SparseChunkHeader;
import com.vendor.loader.sparse.SparseFileAgent;
import com.vendor.loader.sparse.SparseHeader;
import com.vendor.loader.struct.FileSystem;
import com.vendor.loader.struct.UpgradeFileInfo;
import com.vendor.loader.util.FileUtil;
import com.vendor.loader.util.PropertiesUtility;
import com.vendor.loader.util.StringUtil;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;

public class FileHelper {
  public static final String SMALL_FILE_PATH = "loader_PartTemp/";
  
  static final int EMMC_BLOCKSZIE = 512;
  
  static final int SEPARATE_CHUNKLEN = 5242880;
  
  static final int EMPTY_LEN = 5242880;
  
  public static List<UpgradeFileInfo> splitLargeFileToSmallFileList(UpgradeFileInfo largeFileInfo, long filePartSize) throws IOException {
    long largeFileLength = largeFileInfo.getFileLength();
    String largeFileName = largeFileInfo.getFileName();
    List<UpgradeFileInfo> smallFileList = new ArrayList<>();
    if (largeFileLength <= filePartSize) {
      smallFileList.add(largeFileInfo);
      return smallFileList;
    } 
    FileInputStream inputStream = null;
    try {
      byte[] buffer = new byte[2097152];
      int times = 0;
      File file = new File(largeFileName);
      if (file != null && file.exists() && file.isFile())
        inputStream = new FileInputStream(file); 
      if (inputStream == null)
        return null; 
      long address = largeFileInfo.getStartAddress();
      UpgradeFileInfo smallPartInfo = null;
      while (largeFileLength > 0L) {
        File partfile = null;
        int partLength = 0;
        int read = -1;
        String partFilePath = "loader_PartTemp/" + file.getName() + ".part" + System.nanoTime() + "_" + times;
        partfile = new File(partFilePath);
        try {
          partfile.createNewFile();
        } catch (Exception exception) {}
        OutputStream outputtream = null;
        smallPartInfo = largeFileInfo.copy();
        try {
          outputtream = new FileOutputStream(partfile);
          int length = (int)Math.min(filePartSize, buffer.length);
          while (partLength < filePartSize && (
            read = inputStream.read(buffer, 0, length)) > 0) {
            outputtream.write(buffer, 0, read);
            partLength += read;
            if (filePartSize - partLength < buffer.length) {
              length = (int)(filePartSize - partLength);
              continue;
            } 
            length = buffer.length;
          } 
        } finally {
          if (outputtream != null)
            outputtream.close(); 
        } 
        smallPartInfo.setFileName(partfile.getCanonicalPath());
        smallPartInfo.setFileLength(partLength);
        smallPartInfo.setStartAddress(address);
        if (partLength < filePartSize) {
          smallPartInfo.setEndAddress(largeFileInfo
              .getEndAddress());
        } else {
          smallPartInfo.setEndAddress(address + filePartSize);
        } 
        smallFileList.add(smallPartInfo);
        largeFileLength -= partLength;
        address += filePartSize;
        times++;
      } 
    } finally {
      if (inputStream != null)
        try {
          inputStream.close();
        } catch (Exception exception) {} 
    } 
    return smallFileList;
  }
  
  public static long getRealSizeOfSparseFile(File sparseFile, boolean isSparse) throws IOException {
    if (!isSparse)
      return 0L; 
    long partNeedSize = 0L;
    RandomAccessFile inFile = null;
    try {
      inFile = new RandomAccessFile(sparseFile, "r");
      long readLength = 0L;
      byte[] sparseHeaderData = new byte[28];
      inFile.seek(0L);
      int index = inFile.read(sparseHeaderData);
      if (index > -1)
        readLength += 28L; 
      SparseHeader sparseHeader = new SparseHeader();
      sparseHeader.fromBytes(sparseHeaderData);
      for (int i = 0; i < sparseHeader.total_chunks; i++) {
        inFile.seek(readLength);
        byte[] chunkHeaderData = new byte[12];
        int length = inFile.read(chunkHeaderData);
        if (length > -1)
          readLength += 12L; 
        SparseChunkHeader chunkHeader = new SparseChunkHeader();
        chunkHeader.fromBytes(chunkHeaderData);
        long chunkLen = 0L;
        if (chunkHeader.getChunk_type() == -13631) {
          chunkLen = (chunkHeader.getChunk_sz() * sparseHeader.blk_sz);
          partNeedSize += chunkLen;
          int chunkDataSpaceCount = (int)((chunkLen + 5242880L - 1L) / 5242880L);
          int chunkDataSpaceSpace = (int)(chunkLen % 5242880L);
          if (chunkDataSpaceCount == 0) {
            byte[] chunkData = new byte[(int)chunkLen];
            int inFileLength = inFile.read(chunkData);
            if (inFileLength > -1)
              readLength += chunkLen; 
          } else {
            for (int j = 0; j < chunkDataSpaceCount; j++) {
              int len;
              byte[] chunkData = null;
              if (j == chunkDataSpaceCount - 1) {
                if (chunkDataSpaceSpace == 0) {
                  chunkData = new byte[5242880];
                  len = 5242880;
                } else {
                  chunkData = new byte[chunkDataSpaceSpace];
                  len = chunkDataSpaceSpace;
                } 
              } else {
                chunkData = new byte[5242880];
                len = 5242880;
              } 
              int inFileLength = inFile.read(chunkData);
              if (inFileLength > -1)
                readLength += len; 
            } 
          } 
        } else if (chunkHeader.getChunk_type() == -13629) {
          chunkLen = (chunkHeader.getChunk_sz() * sparseHeader.blk_sz);
          partNeedSize += chunkLen;
        } else if (chunkHeader.getChunk_type() == -13630) {
          chunkLen = chunkHeader.getChunk_sz() * sparseHeader.blk_sz;
          partNeedSize += chunkLen;
          readLength += 4L;
        } else {
          throw new IOException(Messages.ValidationSpareseFailed);
        } 
      } 
    } finally {
      if (inFile != null)
        try {
          inFile.close();
        } catch (IOException iOException) {} 
    } 
    return partNeedSize;
  }
  
  public static List<UpgradeFileInfo> splitLargeFileForEMMC(UpgradeFileInfo largeFileInfo, long filePartSize) throws IOException {
    List<UpgradeFileInfo> smallFileList = new ArrayList<>();
    File file = new File(largeFileInfo.getFileName());
    if (file != null) {
      long largeFileLength = file.length();
      if (largeFileLength <= filePartSize) {
        smallFileList.add(largeFileInfo);
        return smallFileList;
      } 
    } 
    boolean isSparse = ((largeFileInfo.getFileSystem() == FileSystem.ext3 || largeFileInfo.getFileSystem() == FileSystem.f2fs || largeFileInfo.getFileSystem() == FileSystem.erofs) && SparseFileAgent.isAndroidSparseFile(file));
    if (isSparse) {
      splitLargeFile(largeFileInfo, filePartSize, smallFileList, file);
    } else {
      smallFileList = splitLargeFileToSmallFileList(largeFileInfo, filePartSize);
    } 
    return smallFileList;
  }
  
  private static void splitLargeFile(UpgradeFileInfo largeFileInfo, long filePartSize, List<UpgradeFileInfo> smallFileList, File file) throws FileNotFoundException, IOException {
    RandomAccessFile inFile = null;
    RandomAccessFile outFile = null;
    long address = largeFileInfo.getStartAddress();
    try {
      inFile = new RandomAccessFile(file, "r");
      File tmpFile = new File("loader_PartTemp/" + file.getName() + System.nanoTime() + ".tmp");
      long readLength = 0L;
      byte[] sparseHeaderData = new byte[28];
      inFile.seek(0L);
      int headerLength = inFile.read(sparseHeaderData);
      if (headerLength < 0)
        return; 
      readLength += 28L;
      SparseHeader sparseHeader = new SparseHeader();
      sparseHeader.fromBytes(sparseHeaderData);
      long partNeedSize = 0L;
      int partBlk = 0;
      FileUtil.deleteFile(tmpFile);
      outFile = new RandomAccessFile(tmpFile, "rw");
      outFile.seek(0L);
      outFile.write(SparseHeader.getEmpty());
      int partChunkNum = 0;
      for (int i = 0; i < sparseHeader.total_chunks; i++) {
        inFile.seek(readLength);
        byte[] chunkHeaderData = new byte[12];
        SparseChunkHeader chunkHeader = new SparseChunkHeader();
        int length = inFile.read(chunkHeaderData);
        if (length > -1) {
          readLength += 12L;
          chunkHeader.fromBytes(chunkHeaderData);
        } 
        long chunkLen = 0L;
        if (chunkHeader.getChunk_type() == -13631) {
          chunkLen = (chunkHeader.getChunk_sz() * sparseHeader.blk_sz);
          partBlk += chunkHeader.getChunk_sz();
          partNeedSize += chunkLen;
          partChunkNum++;
          outFile.write(chunkHeaderData);
          int chunkDataSpaceCount = (int)((chunkLen + 5242880L - 1L) / 5242880L);
          int chunkDataSpaceSpace = (int)(chunkLen % 5242880L);
          if (chunkDataSpaceCount == 0) {
            byte[] chunkData = new byte[(int)chunkLen];
            int inlength = inFile.read(chunkData);
            if (inlength > -1) {
              readLength += chunkLen;
              outFile.write(chunkData);
            } 
          } else {
            for (int j = 0; j < chunkDataSpaceCount; j++) {
              int len;
              byte[] chunkData = null;
              if (j == chunkDataSpaceCount - 1) {
                if (chunkDataSpaceSpace == 0) {
                  chunkData = new byte[5242880];
                  len = 5242880;
                } else {
                  chunkData = new byte[chunkDataSpaceSpace];
                  len = chunkDataSpaceSpace;
                } 
              } else {
                chunkData = new byte[5242880];
                len = 5242880;
              } 
              int index = inFile.read(chunkData);
              if (index > -1) {
                readLength += len;
                outFile.write(chunkData);
              } 
            } 
          } 
        } else if (chunkHeader.getChunk_type() == -13629) {
          chunkLen = (chunkHeader.getChunk_sz() * sparseHeader.blk_sz);
          partBlk += chunkHeader.getChunk_sz();
          partNeedSize += chunkLen;
          partChunkNum++;
          outFile.write(chunkHeaderData);
        } else if (chunkHeader.getChunk_type() == -13630) {
          chunkLen = chunkHeader.getChunk_sz() * sparseHeader.blk_sz;
          byte[] fill_val = new byte[4];
          int len = inFile.read(fill_val, 0, fill_val.length);
          if (len < 0)
            System.out.println("Read Error."); 
          partBlk += chunkHeader.getChunk_sz();
          partNeedSize += chunkLen;
          partChunkNum++;
          outFile.write(chunkHeaderData);
          outFile.write(fill_val);
          readLength += 4L;
        } else {
          break;
        } 
        SparseChunkHeader nextChunkHeader = null;
        if (i < sparseHeader.total_chunks - 1) {
          nextChunkHeader = new SparseChunkHeader();
          byte[] nextChunkHeaderData = new byte[12];
          int inFilelength = inFile.read(nextChunkHeaderData);
          if (inFilelength > -1)
            nextChunkHeader.fromBytes(nextChunkHeaderData); 
        } 
        int measureValue = 0;
        if (nextChunkHeader != null)
          if (nextChunkHeader.getChunk_type() == -13631) {
            measureValue = (int)partNeedSize + nextChunkHeader.getChunk_sz() * sparseHeader.blk_sz;
          } else if (nextChunkHeader.getChunk_type() == -13629) {
            measureValue = (int)partNeedSize + nextChunkHeader.getChunk_sz() * sparseHeader.blk_sz;
          } else if (nextChunkHeader.getChunk_type() == -13630) {
            measureValue = (int)partNeedSize + nextChunkHeader.getChunk_sz() * sparseHeader.blk_sz;
          } else {
            break;
          }  
        boolean isEnough = (measureValue > filePartSize || nextChunkHeader == null);
        if (isEnough) {
          SparseHeader partHeader = new SparseHeader();
          partHeader.magic = sparseHeader.magic;
          partHeader.major_version = sparseHeader.major_version;
          partHeader.minor_version = sparseHeader.minor_version;
          partHeader.file_hdr_sz = 28;
          partHeader.chunk_hdr_sz = 12;
          partHeader.blk_sz = sparseHeader.blk_sz;
          partHeader.total_blks = partBlk;
          partHeader.total_chunks = partChunkNum;
          partHeader.image_checksum = 0;
          outFile.seek(0L);
          outFile.write(partHeader.toBytes());
          UpgradeFileInfo tempInfo = largeFileInfo.copy();
          tempInfo.setFileName(tmpFile.getCanonicalPath());
          tempInfo.setFileLength(tmpFile.length());
          tempInfo.setStartAddress(address);
          if (nextChunkHeader == null) {
            tempInfo.setEndAddress(largeFileInfo
                .getEndAddress());
          } else {
            tempInfo.setEndAddress(address + partNeedSize);
          } 
          smallFileList.add(tempInfo);
          address += partNeedSize;
          partNeedSize = 0L;
          partBlk = 0;
          partChunkNum = 0;
          if (i < sparseHeader.total_chunks - 1) {
            if (i + 1 == sparseHeader.total_chunks - 1) {
              inFile.seek(readLength);
              nextChunkHeader = new SparseChunkHeader();
              byte[] nextChunkHeaderData = new byte[12];
              int inFilelength = inFile.read(nextChunkHeaderData);
              if (inFilelength > -1)
                nextChunkHeader
                  .fromBytes(nextChunkHeaderData); 
              if (nextChunkHeader.getChunk_type() == -13629)
                break; 
            } 
            closeStream(outFile);
            tmpFile = new File("loader_PartTemp/" + file.getName() + System.nanoTime() + ".tmp");
            outFile = outFileWriteData(tmpFile);
          } 
        } 
      } 
    } finally {
      closeStream(outFile);
      closeStream(inFile);
    } 
  }
  
  private static RandomAccessFile outFileWriteData(File tmpFile) throws FileNotFoundException, IOException {
    RandomAccessFile randomAccessFile = null;
    if (tmpFile != null) {
      randomAccessFile = new RandomAccessFile(tmpFile, "rw");
      randomAccessFile.seek(0L);
      randomAccessFile.write(SparseHeader.getEmpty());
    } 
    return randomAccessFile;
  }
  
  private static void closeStream(RandomAccessFile outFile) {
    try {
      if (outFile != null)
        outFile.close(); 
    } catch (Exception exception) {}
  }
  
  public static List<UpgradeFileInfo> getRealBurnFileList(List<UpgradeFileInfo> infoList, long filePartSize) throws IOException {
    List<UpgradeFileInfo> realInfoList = new ArrayList<>();
    for (UpgradeFileInfo info : infoList)
      realInfoList.addAll(splitLargeFileForEMMC(info, filePartSize)); 
    return realInfoList;
  }
  
  public static boolean deleteFile(String rootPath) {
    File file = new File(rootPath);
    if (file == null || !file.exists())
      return false; 
    File[] listFiles = file.listFiles();
    for (File tempFile : listFiles) {
      try {
        if (!tempFile.delete())
          return false; 
      } catch (Exception exception) {}
    } 
    return true;
  }
  
  public static boolean createFile(String rootPath) {
    File rootFile = new File(rootPath);
    if (rootFile != null && !rootFile.exists())
      try {
        if (!rootFile.mkdirs())
          return false; 
      } catch (Exception exception) {} 
    return true;
  }
  
  public static void createTempPartFile() {
    createFile("loader_PartTemp/");
  }
  
  public static void deleteTempPartFile() {
    deleteFile("loader_PartTemp/");
  }
  
  public static boolean checkUpgradeFileInfo(List<UpgradeFileInfo> upgradeInfos) {
    boolean isHaveSelectPartitionFile = false;
    boolean isFileLengthInvalid = false;
    for (UpgradeFileInfo info : upgradeInfos) {
      if (info.isSel()) {
        if (info.getFileName() != null && !info.getFileName().isEmpty())
          isHaveSelectPartitionFile = true; 
        File file = new File(info.getFileName());
        if (file != null && file.exists() && file.length() == 0L)
          isFileLengthInvalid = true; 
      } 
    } 
    if (!isHaveSelectPartitionFile) {
      if (PropertiesUtility.isCmdLine()) {
        Log.errorln(Messages.LoaderComposite_SelectFile);
      } else {
        MessageDialog.openError(Display.getDefault()
            .getActiveShell(), Messages.LoaderComposite_10, Messages.LoaderComposite_SelectFile);
      } 
      return false;
    } 
    if (isFileLengthInvalid) {
      if (PropertiesUtility.isCmdLine()) {
        Log.errorln(Messages.UpgradeInfoGroup_FileIsEmpty);
      } else {
        MessageDialog.openError(Display.getDefault()
            .getActiveShell(), Messages.LoaderComposite_10, Messages.UpgradeInfoGroup_FileIsEmpty);
      } 
      return false;
    } 
    return true;
  }
  
  public static void addFileLenth(List<UpgradeFileInfo> updateFiles) throws IOException {
    UpgradeFileInfo info = null;
    File tempFile = null;
    for (int i = 0; i < updateFiles.size(); i++) {
      info = updateFiles.get(i);
      tempFile = new File(info.getFileName());
      if (tempFile == null || !tempFile.exists() || tempFile
        .isDirectory()) {
        String message = String.format(NLS.bind(Messages.LoaderComposite_InvalidFile, 
              Integer.valueOf(i + 1)), new Object[0]);
        throw new IOException(message);
      } 
      info.setFileLength(tempFile.length());
    } 
  }
  
  public static boolean validateUpdateFilesList(List<UpgradeFileInfo> updateFiles, long part) throws IOException {
    UpgradeFileInfo info = null;
    File tempFile = null;
    for (int i = 0; i < updateFiles.size(); i++) {
      info = updateFiles.get(i);
      tempFile = new File(info.getFileName());
      if (tempFile == null || !tempFile.exists() || tempFile
        .isDirectory()) {
        String message = String.format(NLS.bind(Messages.LoaderComposite_InvalidFile, 
              Integer.valueOf(i + 1)), new Object[0]);
        throw new IOException(message);
      } 
      info.setFileLength(tempFile.length());
      if (info.getLength() < info.getFileLength()) {
        String message = String.format(Messages.LoaderComposite_InvalidFileLength, new Object[] { Integer.valueOf(i + 1) });
        throw new IOException(message);
      } 
      if (info.getFileLength() > part) {
        boolean isSparse = ((info.getFileSystem() == FileSystem.ext3 || info.getFileSystem() == FileSystem.f2fs || info.getFileSystem() == FileSystem.erofs) && SparseFileAgent.isAndroidSparseFile(new File(info
              .getFileName())));
        if (isSparse) {
          long realFileLength = getRealSizeOfSparseFile(new File(info
                .getFileName()), isSparse);
          if (info.getLength() < realFileLength)
            throw new IOException(NLS.bind(Messages.LoaderComposite_InvalidFileLength2, new Object[] { StringUtil.convertUnit(realFileLength), 
                    Integer.valueOf(i + 1) })); 
        } 
      } 
    } 
    return true;
  }
}
